""" Main .py file for running the card-py-bot """
from discord.ext import commands
from card_py_bot import get_card

DESCRIPTION = '''Toasterstein's card-py-bot: An auto magic card link parsing
and embedding Discord bot!'''

BOT = commands.Bot(command_prefix='?', description=DESCRIPTION)


@BOT.event
async def on_ready():
    """ Startup callout/setup """
    print('Logged in as')
    print(BOT.user.id)
    print('------')


@BOT.event
async def on_message(message):
    """ Standard message handler with card and shush functions """
    if "http://gatherer.wizards.com/Pages/Card" in message.content:
        print("likely inputted card url:", message.content)
        card_string = get_card.grab_html_from_url(message.content)

        await BOT.send_message(message.channel, card_string)

    await BOT.process_commands(message)


def main():
    BOT.run('MzIxNzgxNzEzNTAyMDc2OTMw.DBj7Uw.KymkGSfhgfTUqdDkJrEf8tXdCxw')


if __name__ == "__main__":
    main()



""" Handler function for advanced parsing on the wizzards website,
specifically for grabbing both the image data and mana data of a card """
import re
import os
import urllib.request


def read_in_website(input_link):
    """
    Non BeautifulSoup parsing for raw extraction of URL response
    ARGS:

    RETURNS:
    """
    with urllib.request.urlopen(input_link) as response:
        lines = []
        oldline = ""

        for lineprt in response.read():
            lineprt = chr(lineprt)
            if lineprt == '\n':
                oldline = str(oldline)
                if oldline.strip() == "":
                    continue
                else:
                    lines.append(oldline.strip())
                    oldline = ""
            oldline = oldline + lineprt
    return lines


def make_mana_dict():
    mana_dict = dict()
    mana_colors_list = [
        'Green', 'Red', 'Blue', 'Black', 'White',
        'Phyrexian Blue', 'Phyrexian Red', 'Phyrexian Green',
        'Phyrexian Black', 'Phyrexian White',
        '1', '2', '3', '4', '5', '6', '7', '8', '9', '10',
        '11', '12', '13', '14', '15', '16', '1000000',
        'Black or Green', 'Red or White', 'Green or Blue',
        'Blue or Red', 'White or Black', 'White or Blue',
        'Blue or Black', 'Black or Red', 'Red or Green',
        'Green or White', 'Two or White', 'Two or Green',
        'Two or Black', 'Two or Red', 'Two or Blue'
    ]

    config_file = open(os.path.dirname(
        os.path.abspath(__file__)) + '/' + 'mana_config.txt', "r")
    counter = 0

    for mana_emoji_id in config_file.readlines():
        mana_dict[mana_colors_list[counter]] = mana_emoji_id
        counter += 1

    config_file.close()
    return mana_dict


def recuse_mana(start_line_counter, lines):
    mana_tot = []
    mana_dict = make_mana_dict()

    def recuse_mana_r(line_pos, line):
        if line.find("alt=", line_pos) >= 0:
            start_indx = line.find("alt=")
            oldline = ""
            for i in range(start_indx + 5, len(line) - 1):
                if line[i] == '"':
                    try:
                        oldline = mana_dict[oldline].strip()
                    except KeyError:
                        for color_count in range(int(oldline)):
                            mana_tot.append(":white_circle:")
                        break

                    mana_tot.append(oldline)
                    break

                oldline = oldline + line[i]

            recuse_mana_r(0, line[start_indx + 4:])

    div_counter = 0

    for i in range(start_line_counter + 1, len(lines) - 1):
        line = lines[i]

        if line.find('<div') >= 0:
            div_counter += 1
        if line.find('</div>') >= 0:
            div_counter -= 1
        if div_counter == 0:
            break

    recuse_mana_r(0, line)

    mana_string = " ".join(mana_tot)
    return mana_string


def get_img_link(line):
    line = str(line)
    start_indx = line.find('src="')

    line = line[start_indx + 5:]
    end_indx = line.find('"')
    line = line[5:end_indx].strip()

    line = re.sub("amp;", "", line)

    return line


def get_card_mana_string(lines):

    line_counter = 0
    grabbed_mana = False
    img_link = "No image link found"
    for i in range(len(lines)):
        line = lines[i]

        if line.find('Mana Cost:</div>') >= 0 and not grabbed_mana:
            card_mana = recuse_mana(i, lines)
            card_mana = str(card_mana)
            grabbed_mana = True

        if line.find('<div class="cardImage">') >= 0:
            img_link = get_img_link(lines[i + 1])

        line_counter += 1

    return (card_mana, img_link)


def get_mana(url):
    lines = read_in_website(url)
    return get_card_mana_string(lines)



from setuptools import setup


setup(
    name="card-py-bot",
    author="Nathan Klapstein",
    author_email="nklapste@ualberta.ca",
    version="1.4",
    description="A Discord Bot for parsing magic card links",
    url="https://github.com/nklapste/card-py-bot",
    download_url="https://github.com/nklapste/card-py-bot/archive/1.4.tar.gz",
    packages=["card_py_bot"],
    package_data={
        '': ['README.md'],
        'card_py_bot': ['mana_config.txt', 'MANA_ICONS/*.gif'],
    },
    install_requires=[
        'beautifulsoup4',
        'html5lib',
        'discord.py',
    ],
    entry_points={
        'console_scripts': ['card-py-bot = card_py_bot.py_d_bot:main'],
    },
)



""" Handler module for parsing the wizzards website for metadata
on magic cards """

import sys
import re

from urllib.request import urlopen
from bs4 import BeautifulSoup
from card_py_bot import get_mana_colors
import html5lib


def grab_html_from_url(url):
    '''
    ARGS:

    RETURNS:
    '''

    try:
        html = urlopen(url)
    except Exception:
        print("ERROR: CHECK URL")

    soup = BeautifulSoup(html, 'html5lib')

    lines = []
    oldline = ""
    for line_prt in soup.get_text():
        if line_prt == "\n":
            oldline = oldline.strip()
            if oldline == "":
                continue
            lines.append(oldline)
            oldline = ""

        oldline = oldline + line_prt

    try:
        start_indx = lines.index("Card Name:")
        end_indx = lines.index("Artist:")
        lines = lines[start_indx:end_indx]
    except Exception:
        return "Error Parsing Card"

    try:
        name_indx = lines.index("Card Name:")
        card_name = lines[name_indx + 1]
    except Exception:
        card_name = "n/a"

    try:
        c_mana_indx = lines.index("Converted Mana Cost:")
        card_c_mana = lines[c_mana_indx + 1]
    except Exception:
        card_c_mana = "Likely no mana"

    try:
        type_indx = lines.index("Types:")
        card_type = lines[type_indx + 1]
        card_type = re.sub('[^a-zA-z]', " ", card_type)
    except Exception:
        card_type = "n/a"

    try:
        text_indx = lines.index("Card Text:")
        card_text = lines[text_indx + 1]
    except Exception:
        card_text = "n/a"

    try:
        ex_indx = lines.index("Expansion:")
        card_expansion = lines[ex_indx + 1]
    except Exception:
        card_expansion = "n/a"

    try:
        rare_indx = lines.index("Rarity:")
        card_rarity = lines[rare_indx + 1]
    except Exception:
        card_rarity = "n/a"

    try:
        num_indx = lines.index("Card Number:")
        card_number = lines[num_indx + 1]
    except Exception:
        card_number = "n/a"

    try:
        (card_mana, img_link_prt) = get_mana_colors.get_mana(url)
    except Exception as error:
        print(error)
        card_mana = "ERROR: Likely no mana"

    card_string = 'Card Name: "{}"\n'.format(card_name)
    card_string += 'Card Mana: {}\n'.format(card_mana)
    card_string += 'Card Converted Mana: "{}"\n'.format(card_c_mana)
    card_string += 'Card Type: "{}"\n'.format(card_type)
    card_string += 'Card Text: {}\n\n'.format(card_text)

    if img_link_prt == "No image link found":
        card_string += img_link_prt
    else:
        card_string += "http://gatherer.wizards.com/" + img_link_prt
    return card_string


if __name__ == "__main__":
    grab_html_from_url(sys.argv[1])
